import { Command } from '@oclif/command';
import { QueryConfig } from 'pg';
import { Client, QueryResult } from './utils/postgres';
/**
 * Default connection flags for all commands.
 */
export declare const DEFAULT_CONNECTION_FLAGS: any;
/**
 * Connection command flags, default flags present on all commands.
 */
export interface ConnectionCommandFlags {
    host: string;
    port: number;
    dbname: string;
    username: string;
}
/**
 * Table interface.
 */
export interface Table {
    toString(): string;
}
/**
 * Base command for all other commands.
 */
export default abstract class BaseCommand extends Command {
    /**
     * Override getClass method when adding new flags.
     */
    static flags: any;
    private client?;
    /**
     * Returns a query given command args and flag.s
     *
     * @param args Command arguments.
     * @param flags Command flags.
     */
    abstract getQuery(args: any, flags: any): string | QueryConfig;
    beforeRun(args: any, flags: any): void;
    /**
     * Returns a pg client.
     */
    getPgClient(): Client;
    /**
     * Returns a child class.
     */
    getClass(): typeof BaseCommand;
    /**
     * Runs this command.
     */
    run(): Promise<void>;
    /**
     * Logs the result.
     *
     * @param queryResult Query result.
     */
    logResults(queryResult: QueryResult, args: any, flags: any): Promise<void>;
    /**
     * Called before exiting.
     *
     * @param error Error.
     */
    finally(error: Error): Promise<void>;
    /**
     * Returns connection command flags from flag.s
     *
     * @param flags Command flags.
     */
    getConnectionCommandFlags(flags: any): ConnectionCommandFlags;
    /**
     * Connects to a postgres.
     *
     * @param flags Connection flags
     */
    pgConnect(flags: ConnectionCommandFlags): Promise<Client>;
    /**
     * Executes a query.
     *
     * @param client Client.
     * @param queryString Query string or query object.
     */
    pgQuery(client: Client, queryString: string | QueryConfig): Promise<QueryResult>;
    /**
     * Formats a query response to a table.
     *
     * @param response Query response.
     */
    queryResultToTable(response: QueryResult): Table;
}
