"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ascii_table_1 = require("ascii-table");
const moment = require("moment");
const momentDurationFormatSetup = require("moment-duration-format");
const PostgresInterval = require("postgres-interval");
const data_1 = require("./utils/data");
const postgres_1 = require("./utils/postgres");
// Initialize moment duration
momentDurationFormatSetup(moment);
/**
 * Default connection flags for all commands.
 */
exports.DEFAULT_CONNECTION_FLAGS = {
    host: command_1.flags.string({
        char: 'h',
        description: 'database server host',
        helpValue: 'HOST',
        default: postgres_1.DEFAULT_HOST
    }),
    port: command_1.flags.string({
        char: 'p',
        description: 'database server host',
        helpValue: 'PORT',
        default: String(postgres_1.DEFAULT_PORT)
    }),
    dbname: command_1.flags.string({
        char: 'd',
        description: 'database name',
        helpValue: 'DBNAME',
        default: postgres_1.DEFAULT_DBNAME
    }),
    username: command_1.flags.string({
        char: 'U',
        description: 'user name',
        helpValue: 'USERNAME',
        default: postgres_1.DEFAULT_USERNAME
    })
};
/**
 * Base command for all other commands.
 */
class BaseCommand extends command_1.Command {
    // tslint:disable-next-line:no-unused
    beforeRun(args, flags) { }
    /**
     * Returns a pg client.
     */
    getPgClient() {
        if (!this.client) {
            throw new Error('Postgre client not created.');
        }
        return this.client;
    }
    /**
     * Returns a child class.
     */
    getClass() {
        return BaseCommand;
    }
    /**
     * Runs this command.
     */
    async run() {
        const { flags, args } = this.parse(this.getClass());
        const connectionFlags = this.getConnectionCommandFlags(flags);
        this.client = await this.pgConnect(connectionFlags);
        await this.beforeRun(args, flags);
        const queryResult = await this.pgQuery(this.client, this.getQuery(args, flags));
        this.logResults(queryResult, args, flags);
    }
    /**
     * Logs the result.
     *
     * @param queryResult Query result.
     */
    // tslint:disable-next-line:no-unused
    async logResults(queryResult, args, flags) {
        this.log(this.queryResultToTable(queryResult).toString());
    }
    /**
     * Called before exiting.
     *
     * @param error Error.
     */
    async finally(error) {
        super.finally(error);
        if (this.client) {
            await this.client.end();
        }
    }
    /**
     * Returns connection command flags from flag.s
     *
     * @param flags Command flags.
     */
    getConnectionCommandFlags(flags) {
        return {
            host: flags.host,
            port: flags.port,
            dbname: flags.dbname,
            username: flags.username
        };
    }
    /**
     * Connects to a postgres.
     *
     * @param flags Connection flags
     */
    async pgConnect(flags) {
        try {
            return await postgres_1.connect(flags.host, flags.port, flags.dbname, flags.username);
        }
        catch (error) {
            throw new Error('Failed connecting to database: ' + error.message);
        }
    }
    /**
     * Executes a query.
     *
     * @param client Client.
     * @param queryString Query string or query object.
     */
    async pgQuery(client, queryString) {
        try {
            return await postgres_1.query(client, queryString);
        }
        catch (error) {
            throw new Error('Failed executing query: ' + error.message);
        }
    }
    /**
     * Formats a query response to a table.
     *
     * @param response Query response.
     */
    queryResultToTable(response) {
        const heading = response.fields.map((field) => field.name);
        const intervalTypesIndices = {};
        let rows = response.rows.map(row => {
            return heading.map((heading, index) => {
                const value = row[heading] || '';
                if (value instanceof PostgresInterval) {
                    intervalTypesIndices[index] = true;
                    return moment.duration(value.toISO()).format('hh [hrs], mm [min], ss [secs], SSS [millis]');
                }
                return data_1.stripNewLines(value).trim();
            });
        });
        const table = ascii_table_1.factory({ heading, rows });
        if (!rows.length) {
            table.__maxCells = heading.length;
        }
        Object.keys(intervalTypesIndices).forEach(index => table.setAlignRight(index));
        return table;
    }
}
/**
 * Override getClass method when adding new flags.
 */
BaseCommand.flags = Object.assign({}, exports.DEFAULT_CONNECTION_FLAGS);
exports.default = BaseCommand;
