"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const base_command_1 = require("../base-command");
const command_2 = require("../utils/command");
const data_1 = require("../utils/data");
const postgres_1 = require("../utils/postgres");
const COMMAND_NAME = command_2.fileName(__filename);
const DEFAULT_LIMIT = 10;
const DEFAULT_SCHEMA = 'public';
const PG_STAT_STATEMENTS = 'pg_stat_statements';
class OutliersCommand extends base_command_1.default {
    getClass() {
        return OutliersCommand;
    }
    async logResults(_, args, flags) {
        if (flags.reset) {
            this.log(`Statistics on schema ${args.schema} reset successfully.`);
        }
        else {
            super.logResults(_, args, flags);
        }
    }
    getQuery(_, flags) {
        if (flags.reset) {
            return { text: 'select pg_stat_statements_reset()', values: [] };
        }
        const limit = data_1.toInt(flags.limit);
        let truncatedQueryString = flags.truncate
            ? "CASE WHEN length(query) <= 60 THEN query ELSE substr(query, 0, 59) || '…' END"
            : 'query';
        const query = `
      SELECT interval '1 millisecond' * total_time AS total_exec_time,
             to_char((total_time/sum(total_time) OVER()) * 100, 'FM90D0') || '%'  AS prop_exec_time,
             to_char(calls, 'FM999G999G999G990') AS ncalls,
             interval '1 millisecond' * (blk_read_time + blk_write_time) AS sync_io_time,
             ${truncatedQueryString} AS query
      FROM pg_stat_statements WHERE userid = (SELECT usesysid FROM pg_user WHERE usename = current_user LIMIT 1)
      ORDER BY total_time DESC
      LIMIT $1;`;
        return { text: query, values: [limit] };
    }
    async beforeRun(args) {
        const client = this.getPgClient();
        await postgres_1.setActiveSchema(client, args.schema);
        await postgres_1.assertExtensionPresent(client, args.schema, PG_STAT_STATEMENTS);
    }
}
OutliersCommand.description = 'Show queries that have longest execution time in aggregate.';
OutliersCommand.examples = [`${COMMAND_NAME} my-schema -U my-name -t`];
OutliersCommand.usage = [`${COMMAND_NAME} [SCHEMA] [OPTIONS]`];
OutliersCommand.flags = Object.assign({}, base_command_1.DEFAULT_CONNECTION_FLAGS, { reset: command_1.flags.boolean({
        char: 'R',
        description: 'resets the statistics gathered by pg_stat_statements'
    }), truncate: command_1.flags.boolean({
        char: 't',
        description: 'truncate query'
    }), limit: command_1.flags.string({
        char: 'l',
        description: 'limit number of output queries',
        helpValue: 'LIMIT',
        default: String(DEFAULT_LIMIT)
    }) });
OutliersCommand.args = [
    {
        name: 'schema',
        required: false,
        description: 'schema name',
        default: DEFAULT_SCHEMA
    }
];
exports.default = OutliersCommand;
