"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pg_1 = require("pg");
const pgformat = require("pg-format");
const platform_1 = require("./platform");
exports.DEFAULT_HOST = 'localhost';
exports.DEFAULT_PORT = 5432;
exports.DEFAULT_DBNAME = 'postgres';
exports.DEFAULT_USERNAME = platform_1.getUsername();
/**
 * Connects to a postgres database.
 *
 * @param host Host. Default: localhost
 * @param port Port number. Default: 5432
 * @param database Database name. Default: postgres
 * @param user Optional username.
 * @param password Optional password
 * @returns Pg client.
 */
async function connect(host = exports.DEFAULT_HOST, port = exports.DEFAULT_PORT, database = exports.DEFAULT_DBNAME, user = exports.DEFAULT_USERNAME, password) {
    const result = new pg_1.Client({
        user,
        host,
        port,
        database,
        password
    });
    await result.connect();
    await result.query('SELECT 1');
    return result;
}
exports.connect = connect;
/**
 * Executes a query given a client.
 *
 * @param client Client.
 * @param query Query string or query object.
 */
function query(client, query) {
    if (typeof query === 'string') {
        return client.query(query);
    }
    return client.query(query);
}
exports.query = query;
/**
 * Assert specific extension is present and installed.
 *
 * @param client Client.
 * @param schemaName Schema name.
 * @param extensionName Extension name.
 */
async function assertExtensionPresent(client, schemaName, extensionName) {
    const queryString = `SELECT exists(
      SELECT 1 FROM pg_extension e LEFT JOIN pg_namespace n ON n.oid = e.extnamespace
      WHERE e.extname = $1 AND n.nspname = $2
    ) AS available`;
    const result = await client.query(queryString, [extensionName, schemaName]);
    const isAvailable = result.rowCount > 0 && result.rows[0] && result.rows[0].available === true;
    if (!isAvailable) {
        throw new Error(`${extensionName} extension need to be installed on schema ${schemaName}.`);
    }
}
exports.assertExtensionPresent = assertExtensionPresent;
/**
 * Sets the active schema.
 *
 * @param client Client
 * @param schemaName Schema name.
 */
async function setActiveSchema(client, schemaName) {
    await client.query(format('SET search_path TO %L', [schemaName]));
}
exports.setActiveSchema = setActiveSchema;
/**
 * Formats the query string.
 *
 * @param query Query string.
 * @param values Query parts.
 */
function format(query, values) {
    return pgformat(query, ...values);
}
exports.format = format;
